@@ -154,6 +154,10 @@ public:
 	///
 	void addChangesToToc(DocIterator const & cdit, Buffer const & buf,
 	        bool output_active) const;
+	/// set the buffer flag if there are changes in the paragraph
+	void addChangesToBuffer(Buffer const & buf) const;
+	///
+	bool isChangeUpdateRequired() const;
 	///
 	Language const * getParLanguage(BufferParams const &) const;
 	///
