@@ -67,6 +67,7 @@
 #include "support/lassert.h"
 #include "support/lstrings.h"
 #include "support/lyxalgo.h"
+#include "support/lyxtime.h"
 #include "support/textutils.h"
 
 #include <sstream>
@@ -1110,7 +1111,7 @@ bool Text::cursorForwardOneWord(Cursor & cur)
 	Paragraph const & par = cur.paragraph();
 
 	// Paragraph boundary is a word boundary
-	if (pos == lastpos) {
+	if (pos == lastpos || (pos + 1 == lastpos && par.isEnvSeparator(pos))) {
 		if (pit != cur.lastpit())
 			return setCursor(cur, pit + 1, 0);
 		else
@@ -1143,6 +1144,10 @@ bool Text::cursorForwardOneWord(Cursor & cur)
 			     ++pos;
 	}
 
+	// Don't skip a separator inset at the end of a paragraph
+	if (pos == lastpos && pos && par.isEnvSeparator(pos - 1))
+		--pos;
+
 	return setCursor(cur, pit, pos);
 }
 
@@ -1156,8 +1161,14 @@ bool Text::cursorBackwardOneWord(Cursor & cur)
 	Paragraph & par = cur.paragraph();
 
 	// Paragraph boundary is a word boundary
-	if (pos == 0 && pit != 0)
-		return setCursor(cur, pit - 1, getPar(pit - 1).size());
+	if (pos == 0 && pit != 0) {
+		Paragraph & prevpar = getPar(pit - 1);
+		pos = prevpar.size();
+		// Don't stop after an environment separator
+		if (pos && prevpar.isEnvSeparator(pos - 1))
+			--pos;
+		return setCursor(cur, pit - 1, pos);
+	}
 
 	if (lyxrc.mac_like_cursor_movement) {
 		// Skip through punctuation and spaces.
@@ -1600,6 +1611,7 @@ bool Text::erase(Cursor & cur)
 			return dissolveInset(cur);
 
 		if (!par.isMergedOnEndOfParDeletion(cur.buffer()->params().track_changes)) {
+			cur.recordUndo(DELETE_UNDO);
 			par.setChange(cur.pos(), Change(Change::DELETED));
 			cur.forwardPos();
 			needsUpdate = true;
@@ -1890,13 +1902,11 @@ docstring Text::currentState(Cursor const & cur) const
 	Change change = par.lookupChange(cur.pos());
 
 	if (change.changed()) {
-		Author const & a = buf.params().authors().get(change.author);
-		os << _("Change: ") << a.name();
-		if (!a.email().empty())
-			os << " (" << a.email() << ")";
-		// FIXME ctime is english, we should translate that
-		os << _(" at ") << ctime(&change.changetime);
-		os << " : ";
+		docstring const author =
+			buf.params().authors().get(change.author).nameAndEmail();
+		docstring const date = formatted_datetime(change.changetime);
+		os << bformat(_("Changed by %1$s[[author]] on %2$s[[date]]. "),
+		              author, date);
 	}
 
 	// I think we should only show changes from the default
