@@ -443,12 +443,13 @@ bool TextMetrics::redoParagraph(pit_type const pit)
 		if (row_index == pm.rows().size())
 			pm.rows().push_back(Row());
 		Row & row = pm.rows()[row_index];
+		row.pit(pit);
 		row.pos(first);
 		breakRow(row, right_margin, pit);
 		setRowHeight(row, pit);
 		row.setChanged(false);
 		if (row_index || row.endpos() < par.size()
-			|| (row.right_boundary() && par.inInset().lyxCode() != CELL_CODE))
+		    || (row.right_boundary() && par.inInset().lyxCode() != CELL_CODE)) {
 			/* If there is more than one row or the row has been
 			 * broken by a display inset or a newline, expand the text
 			 * to the full allowable width. This setting here is
@@ -458,7 +459,9 @@ bool TextMetrics::redoParagraph(pit_type const pit)
 			 * that, and it triggers when using a caption in a
 			 * longtable (see bugs #9945 and #9757).
 			 */
-			dim_.wid = max_width_;
+			if (dim_.wid < max_width_)
+				dim_.wid = max_width_;
+		}
 		int const max_row_width = max(dim_.wid, row.width());
 		computeRowMetrics(pit, row, max_row_width);
 		first = row.endpos();
@@ -914,7 +917,7 @@ void TextMetrics::breakRow(Row & row, int const right_margin, pit_type const pit
 	// if the row is too large, try to cut at last separator. In case
 	// of success, reset indication that the row was broken abruptly.
 	if (row.shortenIfNeeded(body_pos, width))
-		row.right_boundary(false);
+		row.right_boundary(!row.empty() && row.back().endpos == row.endpos());
 
 	// make sure that the RTL elements are in reverse ordering
 	row.reverseRTL(is_rtl);
@@ -1111,6 +1114,16 @@ pos_type TextMetrics::getPosNearX(Row const & row, int & x,
 	int const xo = origin_.x_;
 	x -= xo;
 
+	int offset = 0;
+	CursorSlice rowSlice(const_cast<InsetText &>(text_->inset()));
+	rowSlice.pit() = row.pit();
+	rowSlice.pos() = row.pos();
+
+	// Adapt to cursor row scroll offset if applicable.
+	if (bv_->currentRowSlice() == rowSlice)
+		offset = bv_->horizScrollOffset();
+	x += offset;
+
 	pos_type pos = row.pos();
 	boundary = false;
 	if (row.empty())
@@ -1164,8 +1177,10 @@ pos_type TextMetrics::getPosNearX(Row const & row, int & x,
 		else
 			boundary = row.right_boundary();
 	}
-	x += xo;
+
+	x += xo - offset;
 	//LYXERR0("getPosNearX ==> pos=" << pos << ", boundary=" << boundary);
+
 	return pos;
 }
 
@@ -1594,7 +1609,8 @@ bool TextMetrics::cursorEnd(Cursor & cur)
 			boundary = true;
 		else
 			--end;
-	}
+	} else if (cur.paragraph().isEnvSeparator(end-1))
+		--end;
 	return text_->setCursor(cur, cur.pit(), end, true, boundary);
 }
 
@@ -1858,7 +1874,6 @@ void TextMetrics::draw(PainterInfo & pi, int x, int y) const
 
 void TextMetrics::drawParagraph(PainterInfo & pi, pit_type const pit, int const x, int y) const
 {
-	BufferParams const & bparams = bv_->buffer().params();
 	ParagraphMetrics const & pm = par_metrics_[pit];
 	if (pm.rows().empty())
 		return;
@@ -1932,7 +1947,7 @@ void TextMetrics::drawParagraph(PainterInfo & pi, pit_type const pit, int const
 
 		// Row signature; has row changed since last paint?
 		if (pi.pain.isDrawingEnabled())
-			row.setCrc(pm.computeRowSignature(row, bparams));
+			row.setCrc(pm.computeRowSignature(row, *bv_));
 		bool row_has_changed = row.changed()
 			|| rowSlice == bv_->lastRowSlice();
 
