@@ -37,6 +37,7 @@ public:
 	FuncCode creationCode() const { return LFUN_FLOAT_INSERT; }
 	void paramsToDialog(Inset const *);
 	docstring dialogToParams() const;
+	bool checkWidgets(bool readonly) const;
 	//@}
 	///
 	void useWide();
@@ -53,15 +54,23 @@ private Q_SLOTS:
 
 private:
 	///
-	void checkAllowed();
+	void checkAllowed() const;
 	///
 	std::string const get(bool & wide, bool & sideways) const;
 	///
 	void initFloatTypeCO(FloatList const & floats);
+	///
+	bool possiblePlacement(char const & p) const;
 
 	/// one of figure or table?
 	bool standardfloat_;
 	///
+	std::string allowed_placement_;
+	///
+	bool allows_wide_;
+	///
+	bool allows_sideways_;
+	///
 	FloatList const * float_list_;
 };
 
