@@ -32,6 +32,7 @@
 #include "KeyMap.h"
 #include "LyX.h"
 #include "LyXRC.h"
+#include "Session.h"
 
 #include "support/debug.h"
 #include "support/gettext.h"
@@ -53,7 +54,8 @@ namespace frontend {
 
 GuiToolbar::GuiToolbar(ToolbarInfo const & tbinfo, GuiView & owner)
 	: QToolBar(toqstr(tbinfo.gui_name), &owner), visibility_(0),
-	  owner_(owner), command_buffer_(0), tbinfo_(tbinfo), filled_(false)
+	  owner_(owner), command_buffer_(0), tbinfo_(tbinfo), filled_(false),
+	  restored_(false)
 {
 	setIconSize(owner.iconSize());
 	connect(&owner, SIGNAL(iconSizeChanged(QSize)), this,
@@ -68,6 +70,22 @@ GuiToolbar::GuiToolbar(ToolbarInfo const & tbinfo, GuiView & owner)
 }
 
 
+void GuiToolbar::setVisible(bool visible)
+{
+	// This is a hack to find out which toolbars have been restored by
+	// MainWindow::restoreState and which toolbars should be initialized 
+	// by us (i.e., new toolbars)
+	restored_ = true;
+	QToolBar::setVisible(visible);
+}
+
+
+bool GuiToolbar::isRestored() const
+{
+	return restored_;
+}
+
+
 void GuiToolbar::fill()
 {
 	if (filled_)
@@ -99,11 +117,11 @@ Action * GuiToolbar::addItem(ToolbarItem const & item)
 	// Get the keys bound to this action, but keep only the
 	// first one later
 	KeyMap::Bindings bindings = theTopLevelKeymap().findBindings(item.func_);
-	if (bindings.size())
+	if (!bindings.empty())
 		text += " [" + toqstr(bindings.begin()->print(KeySequence::ForGui)) + "]";
 
-	Action * act = new Action(&owner_, getIcon(item.func_, false),
-		text, item.func_, text, this);
+	Action * act = new Action(getIcon(item.func_, false),
+				  text, item.func_, text, this);
 	actions_.append(act);
 	return act;
 }
@@ -166,7 +184,7 @@ public:
 
 
 MenuButton::MenuButton(GuiToolbar * bar, ToolbarItem const & item, bool const sticky)
-	: QToolButton(bar), bar_(bar), tbitem_(item), initialized_(false)
+	: QToolButton(bar), bar_(bar), tbitem_(item)
 {
 	setPopupMode(QToolButton::InstantPopup);
 	QString const label = qt_(to_ascii(tbitem_.label_));
@@ -174,31 +192,28 @@ MenuButton::MenuButton(GuiToolbar * bar, ToolbarItem const & item, bool const st
 	setStatusTip(label);
 	setText(label);
 	QString const name = toqstr(tbitem_.name_);
-	QString imagedir = "images/math/";
-	FileName const fname = imageLibFileSearch(imagedir, name, "png");
-	if (fname.exists()) {
-		setIcon(QIcon(getPixmap(imagedir, name, "png")));
-	} else {
-		imagedir = "images/";
-		imageLibFileSearch(imagedir, name, "png");
-		setIcon(QIcon(getPixmap(imagedir, name, "png")));
+	QStringList imagedirs;
+	imagedirs << "images/math/" << "images/";
+	for (int i = 0; i < imagedirs.size(); ++i) {
+		QString imagedir = imagedirs.at(i);
+		FileName const fname = imageLibFileSearch(imagedir, name, "svgz,png",
+			theGuiApp()->imageSearchMode());
+		if (fname.exists()) {
+			setIcon(QIcon(getPixmap(imagedir, name, "svgz,png")));
+			break;
+		}
 	}
 	if (sticky)
 		connect(this, SIGNAL(triggered(QAction *)),
 			this, SLOT(actionTriggered(QAction *)));
 	connect(bar, SIGNAL(iconSizeChanged(QSize)),
 		this, SLOT(setIconSize(QSize)));
+	initialize();
 }
 
-void MenuButton::mousePressEvent(QMouseEvent * e)
+
+void MenuButton::initialize()
 {
-	if (initialized_) {
-		QToolButton::mousePressEvent(e);
-		return;
-	}
-
-	initialized_ = true;
-
 	QString const label = qt_(to_ascii(tbitem_.label_));
 	ButtonMenu * m = new ButtonMenu(label, this);
 	m->setWindowTitle(label);
@@ -216,8 +231,6 @@ void MenuButton::mousePressEvent(QMouseEvent * e)
 		if (!getStatus(it->func_).unknown())
 			m->add(bar_->addItem(*it));
 	setMenu(m);
-
-	QToolButton::mousePressEvent(e);
 }
 
 
@@ -278,7 +291,7 @@ void GuiToolbar::add(ToolbarItem const & item)
 		tb->setToolTip(label);
 		tb->setStatusTip(label);
 		tb->setText(label);
-		InsertTableWidget * iv = new InsertTableWidget(owner_, tb);
+		InsertTableWidget * iv = new InsertTableWidget(tb);
 		connect(tb, SIGNAL(clicked(bool)), iv, SLOT(show(bool)));
 		connect(iv, SIGNAL(visible(bool)), tb, SLOT(setChecked(bool)));
 		connect(this, SIGNAL(updated()), iv, SLOT(updateParent()));
@@ -308,15 +321,13 @@ void GuiToolbar::add(ToolbarItem const & item)
 }
 
 
-void GuiToolbar::update(bool in_math, bool in_table, bool in_review, 
-	bool in_mathmacrotemplate)
+void GuiToolbar::update(int context)
 {
 	if (visibility_ & Toolbars::AUTO) {
-		bool show_it = (in_math && (visibility_ & Toolbars::MATH))
-			|| (in_table && (visibility_ & Toolbars::TABLE))
-			|| (in_review && (visibility_ & Toolbars::REVIEW))
-			|| (in_mathmacrotemplate && (visibility_ & Toolbars::MATHMACROTEMPLATE));
-		setVisible(show_it);
+		bool const was_visible = isVisible();
+		setVisible(visibility_ & context & Toolbars::ALLOWAUTO);
+		if (isVisible() && !was_visible && commandBuffer())
+			commandBuffer()->setFocus();
 	}
 
 	// update visible toolbars only
@@ -358,8 +369,7 @@ void GuiToolbar::restoreSession()
 		settings.value(sessionKey() + "/visibility", error_val).toInt();
 	if (visibility == error_val || visibility == 0) {
 		// This should not happen, but in case we use the defaults
-		LYXERR(Debug::GUI, "Session settings could not be found!. "
-			"Defaults are used instead");
+		LYXERR(Debug::GUI, "Session settings could not be found! Defaults are used instead.");
 		visibility = 
 			guiApp->toolbars().defaultVisibility(fromqstr(objectName()));
 	}
