@@ -110,7 +110,8 @@ public:
 		MATHMACROTEMPLATE = 1024, //< show in math macro template
 		SAMEROW = 2048, //place to the current row, no new line
 		IPA = 4096, //< show when in IPA inset
-		ALLOWAUTO = MATH | TABLE | REVIEW | MATHMACROTEMPLATE | IPA
+		MINIBUFFER = 8192, //< show when command-execute has been invoked
+		ALLOWAUTO = MATH | TABLE | REVIEW | MATHMACROTEMPLATE | IPA | MINIBUFFER
 	};
 
 	typedef std::vector<ToolbarInfo> Infos;
