@@ -28,7 +28,6 @@
 
 #include "support/convert.h"
 #include "support/debug.h"
-#include "support/filetools.h"
 #include "support/foreach.h"
 #include "support/gettext.h"
 #include "support/lstrings.h"
@@ -61,24 +60,24 @@ using namespace lyx::support;
 namespace lyx {
 
 FileName libFileSearch(QString const & dir, QString const & name,
-				QString const & ext)
+				QString const & ext, search_mode mode)
 {
-	return support::libFileSearch(fromqstr(dir), fromqstr(name), fromqstr(ext));
+	return support::libFileSearch(fromqstr(dir), fromqstr(name), fromqstr(ext), mode);
 }
 
 
 FileName imageLibFileSearch(QString & dir, QString const & name,
-				QString const & ext)
+				QString const & ext, search_mode mode)
 {
 	string tmp = fromqstr(dir);
-	FileName fn = support::imageLibFileSearch(tmp, fromqstr(name), fromqstr(ext));
+	FileName fn = support::imageLibFileSearch(tmp, fromqstr(name), fromqstr(ext), mode);
 	dir = toqstr(tmp);
 	return fn;
 }
 
 namespace {
 
-double locstringToDouble(QString const str)
+double locstringToDouble(QString const & str)
 {
 	QLocale loc;
 	bool ok;
@@ -146,7 +145,7 @@ void lengthToWidgets(QLineEdit * input, LengthCombo * combo,
 		combo->setCurrentItem(len.unit());
 		QLocale loc;
 		loc.setNumberOptions(QLocale::OmitGroupSeparator);
-		input->setText(loc.toString(Length(len).value()));
+		input->setText(formatLocFPNumber(Length(len).value()));
 	}
 }
 
@@ -205,6 +204,21 @@ void doubleToWidget(QLineEdit * input, string const & value, char f, int prec)
 }
 
 
+QString formatLocFPNumber(double d)
+{
+	QString result = toqstr(formatFPNumber(d));
+	QLocale loc;
+	result.replace('.', loc.decimalPoint());
+	return result;
+}
+
+
+bool ColorSorter(ColorCode lhs, ColorCode rhs)
+{
+	return compare_no_case(lcolor.getGUIName(lhs), lcolor.getGUIName(rhs)) < 0;
+}
+
+
 void setValid(QWidget * widget, bool valid)
 {
 	if (valid) {
@@ -299,10 +313,10 @@ QStringList texFileList(QString const & filename)
 
 QString const externalLineEnding(docstring const & str)
 {
-#ifdef Q_WS_MACX
+#ifdef Q_OS_MAC
 	// The MAC clipboard uses \r for lineendings, and we use \n
 	return toqstr(subst(str, '\n', '\r'));
-#elif defined(Q_WS_WIN)
+#elif defined(Q_OS_WIN) || defined(Q_CYGWIN_WIN)
 	// Windows clipboard uses \r\n for lineendings, and we use \n
 	return toqstr(subst(str, from_ascii("\n"), from_ascii("\r\n")));
 #else
@@ -458,7 +472,6 @@ Filter::Filter(docstring const & description, string const & globs)
 	string const expanded_globs = convert_brace_glob(globs);
 
 	// Split into individual globs.
-	vector<string> matches;
 	Tokenizer const tokens(expanded_globs, separator);
 	globs_ = vector<string>(tokens.begin(), tokens.end());
 }
@@ -600,6 +613,8 @@ QString guiName(string const & type, BufferParams const & bp)
 		return qt_("Graphics");
 	if (type == "equation")
 		return qt_("Equations");
+	if (type == "external")
+		return qt_("External material");
 	if (type == "footnote")
 		return qt_("Footnotes");
 	if (type == "listing")
@@ -608,6 +623,8 @@ QString guiName(string const & type, BufferParams const & bp)
 		return qt_("Index Entries");
 	if (type == "marginalnote")
 		return qt_("Marginal notes");
+	if (type == "math-macro")
+		return qt_("Math macros");
 	if (type == "nomencl")
 		return qt_("Nomenclature Entries");
 	if (type == "note")
@@ -620,6 +637,8 @@ QString guiName(string const & type, BufferParams const & bp)
 		return qt_("Branches");
 	if (type == "change")
 		return qt_("Changes");
+	if (type == "senseless")
+		return qt_("Senseless");
 	if (prefixIs(type, "index:")) {
 		string const itype = split(type, ':');
 		IndicesList const & indiceslist = bp.indiceslist();
