@@ -16,7 +16,6 @@
 #include "BufferView.h"
 #include "Cursor.h"
 #include "Lexer.h"
-#include "LyXRC.h"
 #include "MetricsInfo.h"
 #include "OutputParams.h"
 #include "RenderPreview.h"
@@ -38,7 +37,6 @@ namespace lyx {
 InsetPreview::InsetPreview(Buffer * buf)
 	: InsetText(buf), preview_(new RenderPreview(this))
 {
-	setAutoBreakRows(true);
 	setDrawFrame(true);
 	setFrameColor(Color_previewframe);
 }
@@ -55,6 +53,18 @@ InsetPreview::InsetPreview(InsetPreview const & other)
 }
 
 
+InsetPreview & InsetPreview::operator=(InsetPreview const & other)
+{
+	if (&other == this)
+		return *this;
+
+	InsetText::operator=(other);
+	preview_.reset(new RenderPreview(*other.preview_, this));
+
+	return *this;
+}
+
+
 void InsetPreview::write(ostream & os) const
 {
 	os << "Preview" << "\n";
@@ -72,7 +82,7 @@ void InsetPreview::addPreview(DocIterator const & inset_pos,
 void InsetPreview::preparePreview(DocIterator const & pos) const
 {
 	TexRow texrow;
-	odocstringstream str;  
+	odocstringstream str;
 	otexstream os(str, texrow);
 	OutputParams runparams(&pos.buffer()->params().encoding());
 	latex(os, runparams);
@@ -102,14 +112,13 @@ void InsetPreview::preparePreview(DocIterator const & pos) const
 		macro_preamble.append(*it);
 
 	docstring const snippet = macro_preamble + str.str();
-	preview_->addPreview(snippet, *pos.buffer());  
+	preview_->addPreview(snippet, *pos.buffer());
 }
 
 
 bool InsetPreview::previewState(BufferView * bv) const
 {
-	if (!editing(bv) && (RenderPreview::status() == LyXRC::PREVIEW_ON ||
-			     RenderPreview::status() == LyXRC::PREVIEW_NO_MATH)) {
+	if (!editing(bv) && RenderPreview::previewText()) {
 		graphics::PreviewImage const * pimage =
 			preview_->getPreviewImage(bv->buffer());
 		return pimage && pimage->image();
