@@ -24,7 +24,6 @@
 #include "Paragraph.h"
 #include "ParagraphParameters.h"
 #include "TextClass.h"
-#include "TexRow.h"
 
 #include "insets/InsetBibitem.h"
 #include "insets/InsetArgument.h"
@@ -58,12 +57,13 @@ enum OpenEncoding {
 struct OutputState
 {
 	OutputState() : open_encoding_(none), cjk_inherited_(0),
-		        prev_env_language_(0)
+		        prev_env_language_(0), open_polyglossia_lang_("")
 	{
 	}
-	int open_encoding_;
+	OpenEncoding open_encoding_;
 	int cjk_inherited_;
 	Language const * prev_env_language_;
+	string open_polyglossia_lang_;
 };
 
 
@@ -88,6 +88,20 @@ string const getPolyglossiaEnvName(Language const * lang)
 }
 
 
+string const getPolyglossiaBegin(string const & lang_begin_command,
+				 string const & lang, string const & opts)
+{
+	string result;
+	if (!lang.empty())
+		result = subst(lang_begin_command, "$$lang", lang);
+	string options = opts.empty() ?
+		    string() : "[" + opts + "]";
+	result = subst(result, "$$opts", options);
+
+	return result;
+}
+
+
 struct TeXEnvironmentData
 {
 	bool cjk_nested;
@@ -115,7 +129,7 @@ static TeXEnvironmentData prepareEnvironment(Buffer const & buf,
 
 	ParagraphList const & paragraphs = text.paragraphs();
 	ParagraphList::const_iterator const priorpit =
-		pit == paragraphs.begin() ? pit : lyx::prev(pit, 1);
+		pit == paragraphs.begin() ? pit : prev(pit, 1);
 
 	OutputState * state = getOutputState();
 	bool const use_prev_env_language = state->prev_env_language_ != 0
@@ -160,17 +174,13 @@ static TeXEnvironmentData prepareEnvironment(Buffer const & buf,
 		if ((lang_end_command.empty() ||
 		    par_lang != doc_lang) &&
 		    !par_lang.empty()) {
-			os << from_ascii(subst(
-				lang_begin_command,
-				"$$lang",
-				par_lang));
-			if (use_polyglossia
-			    && !data.par_language->polyglossiaOpts().empty())
-					os << "["
-					   << from_ascii(data.par_language->polyglossiaOpts())
-					   << "]";
-			  // the '%' is necessary to prevent unwanted whitespace
-			os << "%\n";
+			    string bc = use_polyglossia ?
+					getPolyglossiaBegin(lang_begin_command, par_lang,
+							    data.par_language->polyglossiaOpts())
+				      : subst(lang_begin_command, "$$lang", par_lang);
+			    os << bc;
+			    // the '%' is necessary to prevent unwanted whitespace
+			    os << "%\n";
 		}
 	}
 
@@ -183,6 +193,8 @@ static TeXEnvironmentData prepareEnvironment(Buffer const & buf,
 	}
 
 	if (style.isEnvironment()) {
+		if (par_lang != doc_lang)
+			state->open_polyglossia_lang_ = par_lang;
 		os << "\\begin{" << from_ascii(style.latexname()) << '}';
 		if (!style.latexargs().empty()) {
 			OutputParams rp = runparams;
@@ -223,6 +235,7 @@ static void finishEnvironment(otexstream & os, OutputParams const & runparams,
 			      TeXEnvironmentData const & data)
 {
 	OutputState * state = getOutputState();
+	// BufferParams const & bparams = buf.params(); // FIXME: for speedup shortcut below, would require passing of "buf" as argument
 	if (state->open_encoding_ == CJK && data.cjk_nested) {
 		// We need to close the encoding even if it does not change
 		// to do correct environment nesting
@@ -231,22 +244,21 @@ static void finishEnvironment(otexstream & os, OutputParams const & runparams,
 	}
 
 	if (data.style->isEnvironment()) {
-		os << "\\end{" << from_ascii(data.style->latexname()) << "}\n";
+		os << breakln
+		   << "\\end{" << from_ascii(data.style->latexname()) << "}\n";
 		state->prev_env_language_ = data.par_language;
 		if (runparams.encoding != data.prev_encoding) {
 			runparams.encoding = data.prev_encoding;
-			if (!runparams.isFullUnicode())
-				os << setEncoding(data.prev_encoding->iconvName());
+			os << setEncoding(data.prev_encoding->iconvName());
 		}
 	}
 
 	if (data.leftindent_open) {
-		os << "\\end{LyXParagraphLeftIndent}\n";
+		os << breakln << "\\end{LyXParagraphLeftIndent}\n";
 		state->prev_env_language_ = data.par_language;
 		if (runparams.encoding != data.prev_encoding) {
 			runparams.encoding = data.prev_encoding;
-			if (!runparams.isFullUnicode())
-				os << setEncoding(data.prev_encoding->iconvName());
+			os << setEncoding(data.prev_encoding->iconvName());
 		}
 	}
 
@@ -299,21 +311,6 @@ void TeXEnvironment(Buffer const & buf, Text const & text,
 		// Or     par->params().depth() > current_depth
 		// Or     par->params().leftIndent() != current_left_indent)
 
-		if (par->layout().isParagraph()) {
-			// FIXME (Lgb): How to handle this?
-			//&& !suffixIs(os, "\n\n")
-
-			// (ARRae) There should be at least one '\n' already but we need there to
-			// be two for Standard paragraphs that are depth-increment'ed to be
-			// output correctly. However, tables can also be paragraphs so
-			// don't adjust them.
-
-			// FIXME (Lgb): Will it ever harm to have one '\n' too
-			// many? i.e. that we sometimes will have
-			// three in a row.
-			os << '\n';
-		}
-
 		// FIXME This test should not be necessary.
 		// We should perhaps issue an error if it is.
 		bool const force_plain_layout = text.inset().forcePlainLayout();
@@ -639,6 +636,8 @@ void TeXOnePar(Buffer const & buf,
 
 	// This paragraph's language
 	Language const * const par_language = par.getParLanguage(bparams);
+	Language const * const nextpar_language = nextpar ?
+		nextpar->getParLanguage(bparams) : 0;
 	// The document's language
 	Language const * const doc_language = bparams.language;
 	// The language that was in effect when the environment this paragraph is
@@ -672,22 +671,23 @@ void TeXOnePar(Buffer const & buf,
 		getPolyglossiaEnvName(par_language): par_language->babel();
 	string const prev_lang = use_polyglossia ?
 		getPolyglossiaEnvName(prev_language) : prev_language->babel();
-	string const doc_lang = use_polyglossia ?
-		getPolyglossiaEnvName(doc_language) : doc_language->babel();
 	string const outer_lang = use_polyglossia ?
 		getPolyglossiaEnvName(outer_language) : outer_language->babel();
+	string const nextpar_lang = nextpar_language ? (use_polyglossia ?
+		getPolyglossiaEnvName(nextpar_language) :
+		nextpar_language->babel()) : string();
 	string lang_begin_command = use_polyglossia ?
-		"\\begin{$$lang}" : lyxrc.language_command_begin;
+		"\\begin{$$lang}$$opts" : lyxrc.language_command_begin;
 	string lang_end_command = use_polyglossia ?
 		"\\end{$$lang}" : lyxrc.language_command_end;
 	// the '%' is necessary to prevent unwanted whitespace
 	string lang_command_termination = "%\n";
 
 	// In some insets (such as Arguments), we cannot use \selectlanguage
-	bool const localswitch = !use_polyglossia
-		&& text.inset().forceLocalFontSwitch();
+	bool const localswitch = text.inset().forceLocalFontSwitch();
 	if (localswitch) {
-		lang_begin_command = lyxrc.language_command_local;
+		lang_begin_command = use_polyglossia ?
+			    "\\text$$lang$$opts{" : lyxrc.language_command_local;
 		lang_end_command = "}";
 		lang_command_termination.clear();
 	}
@@ -707,6 +707,8 @@ void TeXOnePar(Buffer const & buf,
 				"$$lang",
 				prev_lang))
 			   << lang_command_termination;
+			if (prev_lang == state->open_polyglossia_lang_)
+				state->open_polyglossia_lang_ = "";
 		}
 
 		// We need to open a new language if we couldn't close the previous
@@ -755,15 +757,10 @@ void TeXOnePar(Buffer const & buf,
 			// With CJK, the CJK tag has to be closed first (see below)
 			if (runparams.encoding->package() != Encoding::CJK
 			    && !par_lang.empty()) {
-				os << from_ascii(subst(
-					lang_begin_command,
-					"$$lang",
-					par_lang));
-				if (use_polyglossia
-				    && !par_language->polyglossiaOpts().empty())
-						os << "["
-						  << from_ascii(par_language->polyglossiaOpts())
-						  << "]";
+				string bc = use_polyglossia ?
+					  getPolyglossiaBegin(lang_begin_command, par_lang, par_language->polyglossiaOpts())
+					  : subst(lang_begin_command, "$$lang", par_lang);
+				os << bc;
 				os << lang_command_termination;
 			}
 		}
@@ -773,6 +770,7 @@ void TeXOnePar(Buffer const & buf,
 	// encoding, since this only affects the position of the outputted
 	// \inputencoding command; the encoding switch will occur when necessary
 	if (bparams.inputenc == "auto"
+		&& !runparams.isFullUnicode() // Xe/LuaTeX use one document-wide encoding  (see also switchEncoding())
 		&& runparams.encoding->package() != Encoding::none) {
 		// Look ahead for future encoding changes.
 		// We try to output them at the beginning of the paragraph,
@@ -834,7 +832,7 @@ void TeXOnePar(Buffer const & buf,
 	bool const useSetSpace = bparams.documentClass().provides("SetSpace");
 	if (par.allowParagraphCustomization()) {
 		if (par.params().startOfAppendix()) {
-			os << "\\appendix\n";
+			os << "\n\\appendix\n";
 		}
 
 		if (!par.params().spacing().isDefault()
@@ -885,8 +883,7 @@ void TeXOnePar(Buffer const & buf,
 			latexArgInsets(par, os, runparams, style.postcommandargs(), "post:");
 		if (runparams.encoding != prev_encoding) {
 			runparams.encoding = prev_encoding;
-			if (!runparams.isFullUnicode())
-				os << setEncoding(prev_encoding->iconvName());
+			os << setEncoding(prev_encoding->iconvName());
 		}
 	}
 
@@ -895,7 +892,7 @@ void TeXOnePar(Buffer const & buf,
 	switch (style.latextype) {
 	case LATEX_ITEM_ENVIRONMENT:
 	case LATEX_LIST_ENVIRONMENT:
-		if (nextpar && (par.params().depth() < nextpar->params().depth()))
+		if (nextpar && par.params().depth() < nextpar->params().depth())
 			pending_newline = true;
 		break;
 	case LATEX_ENVIRONMENT: {
@@ -919,7 +916,13 @@ void TeXOnePar(Buffer const & buf,
 			&& (runparams.isLastPar || !nextpar->hasSameLayout(par))) {
 			if (pending_newline)
 				os << '\n';
-			os << from_ascii(par.params().spacing().writeEnvirEnd(useSetSpace));
+
+			string const endtag =
+				par.params().spacing().writeEnvirEnd(useSetSpace);
+			if (prefixIs(endtag, "\\end{"))
+				os << breakln;
+
+			os << from_ascii(endtag);
 			pending_newline = true;
 		}
 	}
@@ -935,13 +938,12 @@ void TeXOnePar(Buffer const & buf,
 		&& runparams.local_font != 0
 		&& runparams.local_font->isRightToLeft() != par_language->rightToLeft()
 		// are we about to close the language?
-		&&((nextpar && par_language->babel() != (nextpar->getParLanguage(bparams))->babel())
-		   || (runparams.isLastPar && par_language->babel() != outer_language->babel()));
+		&&((nextpar && par_lang != nextpar_lang)
+		   || (runparams.isLastPar && par_lang != outer_lang));
 
 	if (closing_rtl_ltr_environment
 	    || (runparams.isLastPar
-	        && ((!use_polyglossia && par_language->babel() != outer_language->babel())
-		    || (use_polyglossia && par_language->polyglossia() != outer_language->polyglossia())))) {
+	        && par_lang != outer_lang)) {
 		// Since \selectlanguage write the language to the aux file,
 		// we need to reset the language at the end of footnote or
 		// float.
@@ -962,38 +964,46 @@ void TeXOnePar(Buffer const & buf,
 					? getPolyglossiaEnvName(current_language)
 					: current_language->babel();
 				if (!current_lang.empty()) {
-					os << from_ascii(subst(
-						lang_begin_command,
-						"$$lang",
-						current_lang));
+					string bc = use_polyglossia ?
+						    getPolyglossiaBegin(lang_begin_command, current_lang,
+									current_language->polyglossiaOpts())
+						  : subst(lang_begin_command, "$$lang", current_lang);
+					os << bc;
 					pending_newline = !localswitch;
 					unskip_newline = !localswitch;
 				}
 			} else if (!par_lang.empty()) {
-				os << from_ascii(subst(
-					lang_end_command,
-					"$$lang",
-					par_lang));
-				pending_newline = !localswitch;
-				unskip_newline = !localswitch;
+				// If we are in an environment, we have to close the "outer" language afterwards
+				if (!style.isEnvironment() || state->open_polyglossia_lang_ != par_lang) {
+					os << from_ascii(subst(
+						lang_end_command,
+						"$$lang",
+						par_lang));
+					pending_newline = !localswitch;
+					unskip_newline = !localswitch;
+				}
 			}
 		}
 	}
 	if (closing_rtl_ltr_environment)
 		os << "}";
 
+	bool const last_was_separator =
+		par.size() > 0 && par.isEnvSeparator(par.size() - 1);
+
 	if (pending_newline) {
 		if (unskip_newline)
 			// prevent unwanted whitespace
 			os << '%';
-		os << '\n';
+		if (!os.afterParbreak() && !last_was_separator)
+			os << '\n';
 	}
 
 	// if this is a CJK-paragraph and the next isn't, close CJK
 	// also if the next paragraph is a multilingual environment (because of nesting)
 	if (nextpar
 		&& state->open_encoding_ == CJK
-		&& (nextpar->getParLanguage(bparams)->encoding()->package() != Encoding::CJK
+		&& (nextpar_language->encoding()->package() != Encoding::CJK
 		   || (nextpar->layout().isEnvironment() && nextpar->isMultiLingual(bparams)))
 		// inbetween environments, CJK has to be closed later (nesting!)
 		&& (!style.isEnvironment() || !nextpar->layout().isEnvironment())) {
@@ -1031,14 +1041,14 @@ void TeXOnePar(Buffer const & buf,
 	}
 
 	// If this is the last paragraph, and a local_font was set upon entering
-	// the inset, and we're using "auto" or "default" encoding, the encoding
+	// the inset, and we're using "auto" or "default" encoding, and not
+	// compiling with XeTeX or LuaTeX, the encoding
 	// should be set back to that local_font's encoding.
-	// However, do not change the encoding when a fully unicode aware backend
-	// such as XeTeX is used.
 	if (runparams.isLastPar && runparams_in.local_font != 0
 	    && runparams_in.encoding != runparams_in.local_font->language()->encoding()
 	    && (bparams.inputenc == "auto" || bparams.inputenc == "default")
-	    && (!runparams.isFullUnicode())) {
+		&& !runparams.isFullUnicode()
+	   ) {
 		runparams_in.encoding = runparams_in.local_font->language()->encoding();
 		os << setEncoding(runparams_in.encoding->iconvName());
 	}
@@ -1050,20 +1060,52 @@ void TeXOnePar(Buffer const & buf,
 	// we don't need a newline for the last paragraph!!!
 	// Note from JMarc: we will re-add a \n explicitly in
 	// TeXEnvironment, because it is needed in this case
-	if (nextpar) {
+	if (nextpar && !os.afterParbreak() && !last_was_separator) {
+		// Make sure to start a new line
+		os << breakln;
 		Layout const & next_layout = nextpar->layout();
-		if (style == next_layout
-		    // no blank lines before environments!
-		    || !next_layout.isEnvironment()
-		    // unless there's a depth change
-		    // FIXME What we really want to do here is put every \begin and \end
-		    // tag on a new line (which was not the case with nested environments).
-		    // But in the present state of play, we don't have access to the
-		    // information whether the current TeX row is empty or not.
-		    // For some ideas about how to fix this, see this thread:
-		    // http://www.mail-archive.com/lyx-devel@lists.lyx.org/msg145787.html
-		    || nextpar->params().depth() != par.params().depth()) {
-			os << '\n';
+		// A newline '\n' is always output before a command,
+		// so avoid doubling it.
+		if (!next_layout.isCommand()) {
+			// Here we now try to avoid spurious empty lines by
+			// outputting a paragraph break only if: (case 1) the
+			// paragraph style allows parbreaks and no \begin, \end
+			// or \item tags are going to follow (i.e., if the next
+			// isn't the first or the current isn't the last
+			// paragraph of an environment or itemize) and the
+			// depth and alignment of the following paragraph is
+			// unchanged, or (case 2) the following is a
+			// non-environment paragraph whose depth is increased
+			// but whose alignment is unchanged, or (case 3) the
+			// paragraph is not an environment and the next one is a
+			// non-itemize-like env at lower depth, or (case 4) the
+			// paragraph is a command not followed by an environment
+			// and the alignment of the current and next paragraph
+			// is unchanged, or (case 5) the current alignment is
+			// changed and a standard paragraph follows.
+			DocumentClass const & tclass = bparams.documentClass();
+			if ((style == next_layout
+			     && !style.parbreak_is_newline
+			     && !text.inset().getLayout().parbreakIsNewline()
+			     && style.latextype != LATEX_ITEM_ENVIRONMENT
+			     && style.latextype != LATEX_LIST_ENVIRONMENT
+			     && style.align == par.getAlign()
+			     && nextpar->getDepth() == par.getDepth()
+			     && nextpar->getAlign() == par.getAlign())
+			    || (!next_layout.isEnvironment()
+				&& nextpar->getDepth() > par.getDepth()
+				&& nextpar->getAlign() == par.getAlign())
+			    || (!style.isEnvironment()
+				&& next_layout.latextype == LATEX_ENVIRONMENT
+				&& nextpar->getDepth() < par.getDepth())
+			    || (style.isCommand()
+				&& !next_layout.isEnvironment()
+				&& style.align == par.getAlign()
+				&& next_layout.align == nextpar->getAlign())
+			    || (style.align != par.getAlign()
+				&& tclass.isDefaultLayout(next_layout))) {
+				os << '\n';
+			}
 		}
 	}
 
@@ -1105,19 +1147,16 @@ void latexParagraphs(Buffer const & buf,
 		? getPolyglossiaEnvName(bparams.language)
 		: bparams.language->babel();
 	string const lang_begin_command = runparams.use_polyglossia ?
-		"\\begin{$$lang}" : lyxrc.language_command_begin;
+		"\\begin{$$lang}$$opts" : lyxrc.language_command_begin;
 
 	if (maintext && !lyxrc.language_auto_begin &&
 	    !mainlang.empty()) {
 		// FIXME UNICODE
-		os << from_utf8(subst(lang_begin_command,
-					"$$lang",
-					mainlang));
-		if (runparams.use_polyglossia
-		    && !bparams.language->polyglossiaOpts().empty())
-			os << "["
-			    << from_ascii(bparams.language->polyglossiaOpts())
-			    << "]";
+		string bc = runparams.use_polyglossia ?
+			    getPolyglossiaBegin(lang_begin_command, mainlang,
+						bparams.language->polyglossiaOpts())
+			  : subst(lang_begin_command, "$$lang", mainlang);
+		os << bc;
 		os << '\n';
 	}
 
@@ -1206,7 +1245,7 @@ void latexParagraphs(Buffer const & buf,
 	}
 
 	// if "auto end" is switched off, explicitly close the language at the end
-	// but only if the last par is in a babel language
+	// but only if the last par is in a babel or polyglossia language
 	string const lang_end_command = runparams.use_polyglossia ?
 		"\\end{$$lang}" : lyxrc.language_command_end;
 	if (maintext && !lyxrc.language_auto_end && !mainlang.empty() &&
@@ -1215,6 +1254,8 @@ void latexParagraphs(Buffer const & buf,
 					"$$lang",
 					mainlang))
 			<< '\n';
+		if (state->open_polyglossia_lang_ == mainlang)
+			state->open_polyglossia_lang_ = "";
 	}
 
 	// If the last paragraph is an environment, we'll have to close
@@ -1223,6 +1264,14 @@ void latexParagraphs(Buffer const & buf,
 		os << "\\end{CJK}\n";
 		state->open_encoding_ = none;
 	}
+	// Likewise for polyglossia
+	if (maintext && !is_child && state->open_polyglossia_lang_ != "") {
+		os << from_utf8(subst(lang_end_command,
+					"$$lang",
+					state->open_polyglossia_lang_))
+		   << '\n';
+		state->open_polyglossia_lang_ = "";
+	}
 
 	// reset inherited encoding
 	if (state->cjk_inherited_ > 0) {
@@ -1237,6 +1286,13 @@ pair<bool, int> switchEncoding(odocstream & os, BufferParams const & bparams,
 		   OutputParams const & runparams, Encoding const & newEnc,
 		   bool force)
 {
+	// XeTeX/LuaTeX use only one encoding per document:
+	// * with useNonTeXFonts: "utf8plain",
+	// * with XeTeX and TeX fonts: "ascii" (inputenc fails),
+	// * with LuaTeX and TeX fonts: only one encoding accepted by luainputenc.
+	if (runparams.isFullUnicode())
+		return make_pair(false, 0);
+
 	Encoding const & oldEnc = *runparams.encoding;
 	bool moving_arg = runparams.moving_arg;
 	// If we switch from/to CJK, we need to switch anyway, despite custom inputenc
