@@ -37,11 +37,18 @@
 #include "support/qstring_helpers.h"
 
 #include <QDir>
+#include <QTemporaryFile>
 
 #include "support/lassert.h"
 #include "support/regex.h"
 
 #include <fcntl.h>
+#ifdef HAVE_MAGIC_H
+#include <magic.h>
+#endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 
 #include <cerrno>
 #include <cstdlib>
@@ -90,6 +97,60 @@ bool isValidDVIFileName(string const & filename)
 }
 
 
+bool isBinaryFile(FileName const & filename)
+{
+	bool isbinary = false;
+	if (filename.empty() || !filename.exists())
+		return isbinary;
+
+#ifdef HAVE_MAGIC_H
+	magic_t magic_cookie = magic_open(MAGIC_MIME_ENCODING);
+	if (magic_cookie) {
+		bool detected = true;
+		if (magic_load(magic_cookie, NULL) != 0) {
+			LYXERR(Debug::FILES, "isBinaryFile: "
+				"Could not load magic database - "
+				<< magic_error(magic_cookie));
+			detected = false;
+		} else {
+			char const *charset = magic_file(magic_cookie,
+					filename.toFilesystemEncoding().c_str());
+			isbinary = contains(charset, "binary");
+		}
+		magic_close(magic_cookie);
+		if (detected)
+			return isbinary;
+	}
+#endif
+	// Try by looking for binary chars at the beginning of the file.
+	// Note that this is formally not correct, since count_bin_chars
+	// expects utf8, and the passed string can be anything: plain text
+	// in any encoding, or really binary data. In practice it works,
+	// since QString::fromUtf8() drops invalid utf8 sequences, and
+	// while the exact number may not be correct, we still get a high
+	// number for truly binary files.
+
+	ifstream ifs(filename.toFilesystemEncoding().c_str());
+	if (!ifs)
+		return isbinary;
+
+	// Maximum strings to read
+	int const max_count = 50;
+
+	// Maximum number of binary chars allowed
+	int const max_bin = 5;
+
+	int count = 0;
+	int binchars = 0;
+	string str;
+	while (count++ < max_count && !ifs.eof()) {
+		getline(ifs, str);
+		binchars += count_bin_chars(str);
+	}
+	return binchars > max_bin;
+}
+
+
 string const latex_path(string const & original_path,
 		latex_path_extension extension,
 		latex_path_dots dots)
@@ -230,7 +291,7 @@ FileName const fileOpenSearch(string const & path, string const & name,
 // Returns the real name of file name in directory path, with optional
 // extension ext.
 FileName const fileSearch(string const & path, string const & name,
-			  string const & ext, search_mode mode)
+			  string const & exts, search_mode mode)
 {
 	// if `name' is an absolute path, we ignore the setting of `path'
 	// Expand Environmentvariables in 'name'
@@ -239,15 +300,29 @@ FileName const fileSearch(string const & path, string const & name,
 	// search first without extension, then with it.
 	if (fullname.isReadableFile())
 		return fullname;
-	if (ext.empty())
+	if (exts.empty())
 		// We are done.
 		return mode == may_not_exist ? fullname : FileName();
-	// Only add the extension if it is not already the extension of
-	// fullname.
-	if (getExtension(fullname.absFileName()) != ext)
-		fullname = FileName(addExtension(fullname.absFileName(), ext));
-	if (fullname.isReadableFile() || mode == may_not_exist)
-		return fullname;
+	int n = 0;
+	string ext = token(exts, ',', n);
+	while (!ext.empty()) {
+		// Only add the extension if it is not already the extension of
+		// fullname.
+		bool addext = getExtension(fullname.absFileName()) != ext;
+		if (addext) {
+			if (mode == check_hidpi) {
+				FileName fullname2x = FileName(addExtension(fullname.absFileName() + "@2x", ext));
+				if (fullname2x.isReadableFile())
+					return fullname2x;
+			}
+			fullname = FileName(addExtension(fullname.absFileName(), ext));
+		}
+		if (fullname.isReadableFile() || mode == may_not_exist)
+			return fullname;
+		if (addext)
+			fullname.changeExtension("");
+		ext = token(exts, ',', ++n);
+	}
 	return FileName();
 }
 
@@ -257,20 +332,21 @@ FileName const fileSearch(string const & path, string const & name,
 //   2) build_lyxdir (if not empty)
 //   3) system_lyxdir
 FileName const libFileSearch(string const & dir, string const & name,
-			   string const & ext)
+			   string const & ext, search_mode mode)
 {
 	FileName fullname = fileSearch(addPath(package().user_support().absFileName(), dir),
-				     name, ext);
+				     name, ext, mode);
 	if (!fullname.empty())
 		return fullname;
 
 	if (!package().build_support().empty())
 		fullname = fileSearch(addPath(package().build_support().absFileName(), dir),
-				      name, ext);
+				      name, ext, mode);
 	if (!fullname.empty())
 		return fullname;
 
-	return fileSearch(addPath(package().system_support().absFileName(), dir), name, ext);
+	return fileSearch(addPath(package().system_support().absFileName(), dir),
+				      name, ext, mode);
 }
 
 
@@ -325,25 +401,29 @@ FileName const i18nLibFileSearch(string const & dir, string const & name,
 
 
 FileName const imageLibFileSearch(string & dir, string const & name,
-		  string const & ext)
+		  string const & ext, search_mode mode)
 {
 	if (!lyx::lyxrc.icon_set.empty()) {
 		string const imagedir = addPath(dir, lyx::lyxrc.icon_set);
-		FileName const fn = libFileSearch(imagedir, name, ext);
+		FileName const fn = libFileSearch(imagedir, name, ext, mode);
 		if (fn.exists()) {
 			dir = imagedir;
 			return fn;
 		}
 	}
-	return libFileSearch(dir, name, ext);
+	return libFileSearch(dir, name, ext, mode);
 }
 
 
-string const libScriptSearch(string const & command_in, quote_style style)
+string const commandPrep(string const & command_in)
 {
 	static string const token_scriptpath = "$$s/";
+	string const python_call = "python -tt";
 
 	string command = command_in;
+	if (prefixIs(command_in, python_call))
+		command = os::python() + command_in.substr(python_call.length());
+
 	// Find the starting position of "$$s/"
 	string::size_type const pos1 = command.find(token_scriptpath);
 	if (pos1 == string::npos)
@@ -363,6 +443,10 @@ string const libScriptSearch(string const & command_in, quote_style style)
 		// Replace "$$s/" with ""
 		command.erase(pos1, 4);
 	} else {
+		quote_style style = quote_shell;
+		if (prefixIs(command, os::python()))
+			style = quote_python;
+
 		// Replace "$$s/foo/some_script" with "<path to>/some_script".
 		string::size_type const size_replace = size_script + 4;
 		command.replace(pos1, size_replace, quoteName(script, style));
@@ -372,12 +456,35 @@ string const libScriptSearch(string const & command_in, quote_style style)
 }
 
 
+static string createTempFile(QString const & mask)
+{
+	// FIXME: This is not safe. QTemporaryFile creates a file in open(),
+	//        but the file is deleted when qt_tmp goes out of scope.
+	//        Therefore the next call to createTempFile() may create the
+	//        same file again. To make this safe the QTemporaryFile object
+	//        needs to be kept for the whole life time of the temp file name.
+	//        This could be achieved by creating a class TempDir (like
+	//        TempFile, but using a currentlky non-existing
+	//        QTemporaryDirectory object).
+	QTemporaryFile qt_tmp(mask + ".XXXXXXXXXXXX");
+	if (qt_tmp.open()) {
+		string const temp_file = fromqstr(qt_tmp.fileName());
+		LYXERR(Debug::FILES, "Temporary file `" << temp_file << "' created.");
+		return temp_file;
+	}
+	LYXERR(Debug::FILES, "Unable to create temporary file with following template: "
+			<< qt_tmp.fileTemplate());
+	return string();
+}
+
+
 static FileName createTmpDir(FileName const & tempdir, string const & mask)
 {
 	LYXERR(Debug::FILES, "createTmpDir: tempdir=`" << tempdir << "'\n"
 		<< "createTmpDir:    mask=`" << mask << '\'');
 
-	FileName const tmpfl = FileName::tempName(tempdir, mask);
+	QFileInfo tmp_fi(QDir(toqstr(tempdir.absFileName())), toqstr(mask));
+	FileName const tmpfl(createTempFile(tmp_fi.absoluteFilePath()));
 
 	if (tmpfl.empty() || !tmpfl.createDirectory(0700)) {
 		LYXERR0("LyX could not create temporary directory in " << tempdir
@@ -403,7 +510,7 @@ FileName const createLyXTmpDir(FileName const & deflt)
 		// dir inside deflt.
 		return createTmpDir(deflt, "lyx_tmpdir");
 	} else {
-		// some other error occured.
+		// some other error occurred.
 		return createTmpDir(package().system_temp_dir(), "lyx_tmpdir");
 	}
 }
@@ -576,8 +683,8 @@ string const replaceEnvironmentPath(string const & path)
 	// $[A-Za-z_][A-Za-z_0-9]*
 	static string const envvar = "[$]([A-Za-z_][A-Za-z_0-9]*)";
 
-	static regex envvar_br_re("(.*)" + envvar_br + "(.*)");
-	static regex envvar_re("(.*)" + envvar + "(.*)");
+	static regex const envvar_br_re("(.*)" + envvar_br + "(.*)");
+	static regex const envvar_re("(.*)" + envvar + "(.*)");
 	string result = path;
 	while (1) {
 		smatch what;
@@ -593,25 +700,59 @@ string const replaceEnvironmentPath(string const & path)
 
 
 // Return a command prefix for setting the environment of the TeX engine.
-string latexEnvCmdPrefix(string const & path)
+string latexEnvCmdPrefix(string const & path, string const & lpath)
 {
-	if (path.empty() || lyxrc.texinputs_prefix.empty())
+	bool use_lpath = !(lpath.empty() || lpath == "." || lpath == "./");
+
+	if (path.empty() || (lyxrc.texinputs_prefix.empty() && !use_lpath))
 		return string();
 
-	string const texinputs_prefix = os::latex_path_list(
+	string texinputs_prefix = lyxrc.texinputs_prefix.empty() ? string()
+		: os::latex_path_list(
 			replaceCurdirPath(path, lyxrc.texinputs_prefix));
+	string const allother_prefix = os::latex_path_list(path);
 	string const sep = string(1, os::path_separator(os::TEXENGINE));
 	string const texinputs = getEnv("TEXINPUTS");
+	string const bibinputs = getEnv("BIBINPUTS");
+	string const bstinputs = getEnv("BSTINPUTS");
+	string const texfonts = getEnv("TEXFONTS");
+
+	if (use_lpath) {
+		string const abslpath = FileName::isAbsolute(lpath)
+			? os::latex_path(lpath)
+			: os::latex_path(FileName(path + "/" + lpath).realPath());
+		if (texinputs_prefix.empty())
+			texinputs_prefix = abslpath;
+		else if (suffixIs(texinputs_prefix, sep))
+			texinputs_prefix.append(abslpath + sep);
+		else
+			texinputs_prefix.append(sep + abslpath);
+	}
 
 	if (os::shell() == os::UNIX)
 		return "env TEXINPUTS=\"." + sep + texinputs_prefix
-					  + sep + texinputs + "\" ";
+		                           + sep + texinputs + "\" "
+		         + "BIBINPUTS=\"." + sep + allother_prefix
+		                           + sep + bibinputs + "\" "
+		         + "BSTINPUTS=\"." + sep + allother_prefix
+		                           + sep + bstinputs + "\" "
+		         + "TEXFONTS=\"."  + sep + allother_prefix
+		                           + sep + texfonts + "\" ";
 	else
-		// NOTE: the dummy blank dir is necessary to force the
+		// NOTE: the dummy blank dirs are necessary to force the
 		//       QProcess parser to quote the argument (see bug 9453)
 		return "cmd /d /c set \"TEXINPUTS=." + sep + " "
-						+ sep + texinputs_prefix
-						+ sep + texinputs + "\" & ";
+		                                + sep + texinputs_prefix
+		                                + sep + texinputs + "\" & "
+		               + "set \"BIBINPUTS=." + sep + " "
+		                                + sep + allother_prefix
+		                                + sep + bibinputs + "\" & "
+		               + "set \"BSTINPUTS=." + sep + " "
+		                                + sep + allother_prefix
+		                                + sep + bstinputs + "\" & "
+		               + "set \"TEXFONTS=."  + sep + " "
+		                                + sep + allother_prefix
+		                                + sep + texfonts + "\" & ";
 }
 
 
@@ -642,7 +783,7 @@ string const replaceCurdirPath(string const & path, string const & pathlist)
 		}
 		if (i != string::npos) {
 			newpathlist += sep;
-			// Stop here if the last element is empty 
+			// Stop here if the last element is empty
 			if (++i == oldpathlist.length())
 				break;
 		}
@@ -769,6 +910,8 @@ string const unzippedFileName(string const & zipped_file)
 	string const ext = getExtension(zipped_file);
 	if (ext == "gz" || ext == "z" || ext == "Z")
 		return changeExtension(zipped_file, string());
+	else if (ext == "svgz")
+		return changeExtension(zipped_file, "svg");
 	return onlyPath(zipped_file) + "unzipped_" + onlyFileName(zipped_file);
 }
 
@@ -818,13 +961,11 @@ docstring const makeDisplayPath(string const & path, unsigned int threshold)
 	if (dstr.empty()) {
 		// Yes, filename itself is too long.
 		// Pick the start and the end of the filename.
-		dstr = from_utf8(onlyFileName(path));
-		docstring const head = dstr.substr(0, threshold / 2 - 3);
-
-		docstring::size_type len = dstr.length();
-		docstring const tail =
-			dstr.substr(len - threshold / 2 - 2, len - 1);
-		dstr = head + from_ascii("...") + tail;
+		docstring fstr = from_utf8(onlyFileName(path));
+		dstr = fstr;
+		if (support::truncateWithEllipsis(dstr, threshold / 2))
+			dstr += fstr.substr(fstr.length() - threshold / 2 - 2,
+								docstring::npos);
 	}
 
 	return from_utf8(os::external_path(prefix + to_utf8(dstr)));
@@ -837,7 +978,7 @@ bool readLink(FileName const & file, FileName & link)
 	string const encoded = file.toFilesystemEncoding();
 #ifdef HAVE_DEF_PATH_MAX
 	char linkbuffer[PATH_MAX + 1];
-	int const nRead = ::readlink(encoded.c_str(),
+	ssize_t const nRead = ::readlink(encoded.c_str(),
 				     linkbuffer, sizeof(linkbuffer) - 1);
 	if (nRead <= 0)
 		return false;
@@ -851,7 +992,7 @@ bool readLink(FileName const & file, FileName & link)
 		if (nRead < 0) {
 			return false;
 		}
-		if (nRead < buf.size() - 1) {
+		if (static_cast<size_t>(nRead) < buf.size() - 1) {
 			break;
 		}
 		buf.resize(buf.size() * 2);
@@ -883,7 +1024,6 @@ cmd_ret const runCommand(string const & cmd)
 	// variants ipstream, opstream
 
 #if defined (_WIN32)
-	int fno;
 	STARTUPINFO startup;
 	PROCESS_INFORMATION process;
 	SECURITY_ATTRIBUTES security;
@@ -925,7 +1065,7 @@ cmd_ret const runCommand(string const & cmd)
 				0, 0, &startup, &process)) {
 
 			CloseHandle(process.hThread);
-			fno = _open_osfhandle((long)in, _O_RDONLY);
+			int fno = _open_osfhandle((long)in, _O_RDONLY);
 			CloseHandle(out);
 			inf = _fdopen(fno, "r");
 		}
@@ -1069,19 +1209,43 @@ bool prefs2prefs(FileName const & filename, FileName const & tempfile, bool lfun
 	return true;
 }
 
+
+bool configFileNeedsUpdate(string const & file)
+{
+	// We cannot initialize configure_script directly because the package
+	// is not initialized yet when static objects are constructed.
+	static FileName configure_script;
+	static bool firstrun = true;
+	if (firstrun) {
+		configure_script =
+			FileName(addName(package().system_support().absFileName(),
+				"configure.py"));
+		firstrun = false;
+	}
+
+	FileName absfile =
+		FileName(addName(package().user_support().absFileName(), file));
+	return !absfile.exists()
+		|| configure_script.lastModified() > absfile.lastModified();
+}
+
+
 int fileLock(const char * lock_file)
 {
 	int fd = -1;
 #if defined(HAVE_LOCKF)
 	fd = open(lock_file, O_CREAT|O_APPEND|O_SYNC|O_RDWR, 0666);
+	if (fd == -1)
+		return -1;
 	if (lockf(fd, F_LOCK, 0) != 0) {
 		close(fd);
-		return(-1);
+		return -1;
 	}
 #endif
-	return(fd);
+	return fd;
 }
 
+
 void fileUnlock(int fd, const char * /* lock_file*/)
 {
 #if defined(HAVE_LOCKF)
